﻿using Vintagestory.API.Client;
using Vintagestory.API.Common;
using Vintagestory.API.Config;
using Vintagestory.API.MathTools;
using Vintagestory.GameContent;

namespace nrw.frese.stonepile.item
{
    public class ItemPilableStone : ItemStone
    {
        public override void OnHeldInteractStart(ItemSlot itemslot, EntityAgent byEntity, BlockSelection blockSel, EntitySelection entitySel, bool firstEvent, ref EnumHandHandling handling)
        {
            if(blockSel != null)
            {
                IPlayer byPlayer = (byEntity as EntityPlayer).Player;
                IWorldAccessor world = byEntity.World;
                BlockStonePile stonepileBlock = world.GetBlock(new AssetLocation("stonepiles:stonepile-" + Variant["rock"])) as BlockStonePile;
                BlockPos pos = blockSel.Position;
                Block selectedBlock = world.BlockAccessor.GetBlock(pos);

                if (stonepileBlock != null && selectedBlock.BlockId != stonepileBlock.BlockId)
                {
                    if (byEntity.Controls.Sprint)
                    {
                        if (stonepileBlock == null) return;
                        BlockPos blockPos = pos.Copy();
                        if (byEntity.World.BlockAccessor.GetBlock(blockPos).Replaceable < 6000) blockPos.Add(blockSel.Face);

                        bool ok = stonepileBlock.Construct(itemslot, byEntity.World, blockPos, byPlayer);

                        Cuboidf[] collisionBoxes = byEntity.World.BlockAccessor.GetBlock(blockPos).GetCollisionBoxes(byEntity.World.BlockAccessor, blockPos);

                        if (collisionBoxes != null && collisionBoxes.Length > 0 && CollisionTester.AabbIntersect(collisionBoxes[0], blockPos.X, blockPos.Y, blockPos.Z, byPlayer.Entity.CollisionBox, byPlayer.Entity.SidedPos.XYZ))
                        {
                            byPlayer.Entity.SidedPos.Y += collisionBoxes[0].Y2 - (byPlayer.Entity.SidedPos.Y - (int)byPlayer.Entity.SidedPos.Y);
                        }

                        if (ok)
                        {
                            handling = EnumHandHandling.PreventDefaultAction;
                            //((byEntity as EntityPlayer).Player as IClientPlayer).TriggerFpAnimation(EnumHandInteract.HeldItemInteract);
                        }
                        else
                        {
                            base.OnHeldInteractStart(itemslot, byEntity, blockSel, entitySel, firstEvent, ref handling);
                        }

                        handling = EnumHandHandling.PreventDefault;
                        byEntity.Attributes.SetInt("aimingCancel", 1);
                        return;
                    }
                    else
                    {
                        base.OnHeldInteractStart(itemslot, byEntity, blockSel, entitySel, firstEvent, ref handling);
                    }
                }
                else
                {
                    //Handle stonepile action
                }
            }
            else
            {
                base.OnHeldInteractStart(itemslot, byEntity, blockSel, entitySel, firstEvent, ref handling);
            }
        }
    }
}
